<?php

use Symfony\Component\HttpKernel\Kernel;
use Symfony\Component\Config\Loader\LoaderInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\HttpKernelInterface;
use Symfony\Component\Yaml\Yaml;

class AppKernel extends Kernel {

    public function registerBundles() {

        $bundles = array(
            new Symfony\Bundle\FrameworkBundle\FrameworkBundle(),
            new Symfony\Bundle\SecurityBundle\SecurityBundle(),
            new Symfony\Bundle\TwigBundle\TwigBundle(),
            new Symfony\Bundle\MonologBundle\MonologBundle(),
            new Symfony\Bundle\SwiftmailerBundle\SwiftmailerBundle(),
            new Symfony\Bundle\AsseticBundle\AsseticBundle(),
            new Doctrine\Bundle\DoctrineBundle\DoctrineBundle(),
            new Doctrine\Bundle\MigrationsBundle\DoctrineMigrationsBundle(),
            new Doctrine\Bundle\FixturesBundle\DoctrineFixturesBundle(),
            new Sensio\Bundle\FrameworkExtraBundle\SensioFrameworkExtraBundle(),
            new Knp\Bundle\PaginatorBundle\KnpPaginatorBundle(),
            new Stof\DoctrineExtensionsBundle\StofDoctrineExtensionsBundle(),
            new Dj3c1t\AppBundle\Dj3c1tAppBundle(),
            new Dj3c1t\SourceMLBundle\Dj3c1tSourceMLBundle(),
            new Dj3c1t\JQFileUploadBundle\Dj3c1tJQFileUploadBundle(),
            new Stfalcon\Bundle\TinymceBundle\StfalconTinymceBundle(),
            new Dj3c1t\HttpBundle\Dj3c1tHttpBundle(),
        );

        if (in_array($this->getEnvironment(), array('dev', 'test'))) {
            $bundles[] = new Symfony\Bundle\DebugBundle\DebugBundle();
            $bundles[] = new Symfony\Bundle\WebProfilerBundle\WebProfilerBundle();
            $bundles[] = new Sensio\Bundle\DistributionBundle\SensioDistributionBundle();
            $bundles[] = new Sensio\Bundle\GeneratorBundle\SensioGeneratorBundle();
        }

        return $bundles;

    }

    public function registerContainerConfiguration(LoaderInterface $loader) {

        $loader->load($this->getRootDir().'/config/config_'.$this->getEnvironment().'.yml');

    }

    public function handle(Request $request, $type = HttpKernelInterface::MASTER_REQUEST, $catch = true) {
        try {
            $webDir = $this->getWebDir();
        }
        catch(\Exception $e) {
            return new Response($e->getMessage());
        }
        $symfony_dir = dirname($this->getRootDir());
        if(
                !is_writable($symfony_dir."/app/cache")
            ||  !is_writable($symfony_dir."/app/logs")
            ||  !is_writable($symfony_dir."/".$webDir)
        ) {
            $content =
                "<p>some files or directories aren't writable</p>"
                ."<ul>";
            if(!is_writable($symfony_dir."/app/cache")) {
                $content .= "<li>app/cache</li>";
            }
            if(!is_writable($symfony_dir."/app/logs")) {
                $content .= "<li>app/logs</li>";
            }
            if(!is_writable($symfony_dir."/".$webDir)) {
                $content .= "<li>".$webDir."</li>";
            }
            $content .= "</ul>";
            return new Response($content);
        }
        if(substr($request->getUri(), -23) == "import/previous_version") {
            require_once $symfony_dir."/".$webDir."/previous.version.pathes.php";
            require_once $symfony_dir."/".$webDir."/".$PATHES["mw_path"]."/mw_app.php";
            $app = new mw_app($symfony_dir."/".$webDir."/previous.version.pathes.php");
            if(!$app->init()) {
                return new Response("impossible d'initialiser l'application");
            }
            $request->attributes->set("mw_app", $app);
        }
        return parent::handle($request, $type, $catch);
    }

    public function getWebDir() {
        $parameters_file = $this->getRootDir().'/config/parameters.yml';
        if(!@file_exists($parameters_file)) {
            throw new \Exception("missing parameters.yml");
        }
        if(!($content = @file_get_contents($parameters_file))) {
            throw new \Exception("can't read paramerters.yml");
        }
        try {
            $yaml = Yaml::parse($content);
        }
        catch(\Exception $e) {
            throw new \Exception("invalid paramerters.yml");
        }
        if(!isset($yaml['parameters']['web_dir'])) {
            throw new \Exception("missing paramerter web_dir in parameters.yml");
        }
        return $yaml['parameters']['web_dir'];
    }

}
